# https://stackoverflow.com/questions/12459811/how-to-embed-matplotlib-in-pyqt-for-dummies
# https://stackoverflow.com/questions/18625085/how-to-plot-a-wav-file

import sys
import numpy

from PySide2 import QtGui,QtWidgets
from PySide2.QtWidgets import QGroupBox
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure

from stopeight.util.editor.runnable import EditorApp

from stopeight.util.editor.data import WaveData
from stopeight.util.editor import runnable

class WaveArea(QtWidgets.QDockWidget):
    def __init__(self, parent=EditorApp().window):
        super(WaveArea, self).__init__(parent)

        # a figure instance to plot on
        self.figure = Figure()

        # this is the Canvas Widget that displays the `figure`
        # it takes the `figure` instance as a parameter to __init__
        self.canvas = FigureCanvas(self.figure)

        # this is the Navigation widget
        # it takes the Canvas widget and a parent
        self.plotbar = NavigationToolbar(self.canvas, self)

        # Just some button connected to `plot` method
        #self.button = QtWidgets.QPushButton('Plot')
        #self.button.clicked.connect(self.plot)

        # set the layout
        #layout = QtWidgets.QVBoxLayout()
        #layout.addWidget(self.toolbar)
        #layout.addWidget(self.canvas)
        #layout.addWidget(self.button)
        #self.setLayout(layout)

        self.ax=None

        self.data=WaveData(shape=(1,),dtype=numpy.int16)
        self.sr = 44100

    def __call__(self,data,clear=True):
        if clear:
            self.clearImage()
        self.plot(data)
    __call__.__annotations__ = {'data': WaveData}


    def identify(self):
        return str(self.sr)

    def clearImage(self):
        # discards the old graph
        if self.ax != None:
            self.ax.clear()

    def plot(self, samples, **kwargs):
        ''' plot some random stuff '''
        # create an axis
        self.ax = self.figure.add_subplot(111)
        #self.ax.figure(1)

        #ax.title("Signal Wave...")
        self.ax.plot(samples)

        #self.ax.set_ylabel([])
        #self.ax.set_xlabel([])

        # Turn off tick labels
        #self.ax.set_yticklabels([])
        #self.ax.set_xticklabels([])

        #self.ax.show()
        # refresh canvas
        self.canvas.draw()



import sys
from PySide2.QtWidgets import QApplication,QMainWindow
from PySide2.QtCore import Qt

class EditorWindow(QMainWindow):
    def __init__(self, *args, **kwargs):
        s = super().__init__(*args, **kwargs)
        self.setWindowTitle("Editor")

# Singleton
class EditorApp(QApplication):
    window=None
    ## object.__call__ is object()(), not object() unlike type()
    def __new__(cls, *args, **kwargs):
        if cls.instance()==None:
            _instance = super().__new__(cls,sys.argv)
        else:
            _instance = cls.instance()
        _instance.setAttribute(Qt.AA_CompressHighFrequencyEvents, False)
        _instance.setAttribute(Qt.AA_CompressTabletEvents, False)
        _instance.setAttribute(Qt.AA_SynthesizeTouchForUnhandledMouseEvents, False)
        _instance.setAttribute(Qt.AA_SynthesizeMouseForUnhandledTouchEvents, False)
        _instance.setAttribute(Qt.AA_SynthesizeMouseForUnhandledTabletEvents, False)
        return _instance
    def __init__(self, *args, **kwargs):
        if self.window==None:
            super().__init__(sys.argv)
            self.window = EditorWindow()



#!/usr/bin/env python

import os
#cmake start
__import__('cmake')
from cmake import CMakeExtension, CMakeBuild
#cmake end

from setuptools import setup, Extension, find_namespace_packages

setup( use_scm_version=True,
       packages=find_namespace_packages(include=['stopeight.*']),
#cmake start
#Hack for faster compile. --inplace and copy_extensions_to_source to working anyway
#       ext_modules=[CMakeExtension('stopeight.matrix',''),CMakeExtension('stopeight.grapher',''),CMakeExtension('stopeight.analyzer',''),CMakeExtension('stopeight.legacy','')],
       ext_modules=[CMakeExtension('stopeight.stopeight-clibs','')],
       cmdclass=dict(build_ext=CMakeBuild),
#cmake end
)



# Copyright (C) 2017 Fassio Blatter

class Singleton(type):
    """
    inherit: (StatefulClass,metaclass=Singleton)
    not suitable for StatefulClasses with custom new or init
    not suitable for StatefulClasses with their own type information
    """
    _instances = {}
    ## type.__call__ is type(), not type()() unlike object()()
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton,cls).__call__(*args, **kwargs)
        return cls._instances[cls]
    ## Prohibited!
    # def __new__(cls, *args, **kwargs):
    #     print(str(cls)+"__new__")
    #     super(Singleton,cls).__new__(cls,*args, **kwargs)
    def __init__(cls, *args, **kwargs):
        super(Singleton,cls).__init__(*args, **kwargs)

class SingletonList(list,metaclass=Singleton):
    pass
import numpy
from numpy import ndarray
class SingletonNumpy(ndarray,metaclass=Singleton):
    def __call__(cls, *args, **kwargs):
        print("__call__ sub")

if __name__ == '__main__':
    print("__main__")
    test1 = SingletonList()
    test1.append("5")
    print(test1)
    if (len(test1)!=1):
        raise Exception("test1 assignment failed!")
    test2 = SingletonList()
    test2.append("7")
    print(test2)
    if (len(test2)!=2):
        raise Exception("test2 assignment failed!")
    if (len(test1)!=len(test2)):
        raise Exception("SingletonList instantiation test failed!")
    test3 = SingletonNumpy(shape=(1,),dtype=numpy.int16)
    test3[0]=77
    print(test3)
    if (len(test3)!=1):
        raise Exception("test3 assignment failed!")
    test4 = SingletonNumpy(shape=(1,),dtype=numpy.int16)
    test4 = numpy.append(test4,[88])
    print(test4)
    if (len(test4)!=2):
        raise Exception("test4 assignment failed!")
    if (len(test3)!=len(test4)-1):
        print(str(len(test3))+","+str(len(test4)))
        raise Exception("SingletonList instantiation test failed!")




#!/usr/bin/env python

# Copyright (C) 2009-2012 Specific Purpose Software GmbH

import numpy
from stopeight.comparator import vectorTools
#from stopeight.comparator.shapeMatch import ShapeMatchClass
from stopeight.comparator.shapeMatchSubSection import ShapeMatchSubSectionClass
import multiprocessing
from stopeight.comparator.vectorTools import NumpyLine

import stopeight.logging as log
log.disable(log.CRITICAL)

class mp_data_line():
    def __init__(self,input,dbrecord,listpos):
        if (isinstance(input, numpy.ndarray) and isinstance(dbrecord, numpy.ndarray)):
            self.input_line=input
            self.record_line=dbrecord
            self.record_id=listpos
#            self.record_id=dbrecord.id
            log.debug('mp_data_line():'+'record'+str(dbrecord)+'added')
        else:
            log.error('mp_data_line(): Numpy array expected')

def _mp_match_func_name(nline):
    '''
    ShapeMatchSubSectionClass: unweightedReverseSubSection
    ShapeMatchClass: unweightedReverse
    '''
    return ShapeMatchSubSectionClass(nline)

def mp_match_line(next_record,results):
    counter = 0
    while 1:
        try:
            data=next_record.get(False)
        except:
            break
        counter += 1
        Geometry = _mp_match_func_name(data.record_line.view(NumpyLine))
        if Geometry.match(data.input_line.view(NumpyLine)):
            results.append((data.record_id))

class MPLine():
    allLines = []
    def __init__(cls, lines):
        if isinstance(lines, list) and all(isinstance(item,numpy.ndarray) for item in lines):
            cls.allLines = lines
            log.debug('MPLine.init():'+str(len(lines))+'records added')
        else:
            log.error('MPLine.init(): list of Numpy arrays please '+str(type(lines)))
    
    def matchLine(self,line):
        if (isinstance(line, numpy.ndarray)):
            manager = multiprocessing.Manager()
            log.debug('MPLine.matchLine(): Manager initialized')

            results = manager.list()

            comparison = line
    #        comparison = comparison.view(dtype=numpy.dtype(float))
            log.debug('comparing %s:'%(comparison))
            next_record = multiprocessing.Queue()
            log.debug('MPLine.matchLine(): Queue created')
            log.debug('MPLine.matchLine():'+'initializing'+str(len(self.allLines))+'records')
            for n,dbrecord in enumerate(self.allLines):
                if not isinstance(dbrecord, numpy.ndarray):
                    log.error('MPLine.matchLine(): allLines(): wrong format '+str(type(dbrecord)))
                    break
                log.debug('to %s'%(dbrecord))
    #            dbrecord = dbrecord.view(dtype=numpy.dtype(float),type=numpy.ndarray)
                next_record.put(mp_data_line(comparison,dbrecord,n))
            pool = []
            for i in range(0,multiprocessing.cpu_count()): #number of CPUs plus 1
                process = multiprocessing.Process(target=mp_match_line, args=(next_record,results))
                pool.append(process)
                process.start()
            for proc in pool:
                proc.join()
            for r in results:
                log.info('MPLine.matchLine(): found '+str(r))
            manager.shutdown
            log.debug('MPLine.matchLine(): Manager shutdown')
            return results
        else:
            log.warning('MPLine.matchLine(): is not Numpy array')
            log.error('MPLine.matchLine():'+str(type(line))+'unexpected')

def run(dbrecords,line):
    matcher = MPLine(dbrecords)
    log.info('sending job to pooling')
    return matcher.matchLine(line)

def runNPopulate(dbrecords,line):
    result = run(dbrecords,line)
    return populate(result)

def populate(result):
    results = []
    for pos in result:
        log.info('matched line position '+str(pos))
        results.append(dbrecords[pos])
    return results
    
    
if __name__ == "__main__":
    log.setLevel(log.NOTSET)
    log.info('Starting lineMatch...')
    s = numpy.array([[20,20],[55,130],[80,60],[120,85],[200,10]])
    #log.info('rendering input.svg')
    #input.printVectors('input')
    dbrecord = numpy.array([[13,40],[50,125],[80,61],[123,79],[191,11]])
    #log.info('rendering dbrecord.svg')
    #dbrecord.printVectors('dbrecord')
    dbrecords = []
    dbrecords.append(dbrecord)
    res = runNPopulate(dbrecords,s)    
    if len(res)>0:
        log.info('Hooray! '+str(len(res))+'results')



def test_comparator():
    from stopeight.util.parser import process_directory
    from stopeight import legacy
    from stopeight.multiprocessing import pooling
    from numpy import array
    import os
    listoflines = process_directory(os.path.join('stopeight-clibs','legacy','test-data'),'.sp',legacy.parse_file,legacy.stroke_sequential)
    listofarrays = []
    for line in listoflines:
        listofarrays.append(array(line))
    comparator = pooling.MPLine(listofarrays)
    for i,line in enumerate(listofarrays):
        matches = comparator.matchLine(line)
        print('Line ' + str(i) + ' matched ' + str(len(matches)) + ' occurences ' + str(matches))


# Copyright (C) 2019 Fassio Blatter
from stopeight import analyzer
version=analyzer.version

from stopeight.util.editor.data import ScribbleData

def legal_segments(data):
    from stopeight.matrix import Vectors
    from stopeight.analyzer import legal_segments
    return legal_segments(Vectors(data)).__array__().view(ScribbleData)
legal_segments.__annotations__ = {'data':ScribbleData,'return':ScribbleData}




import inspect
import os

def _path(stack):
    # bug: PERFORMANCE
    return os.path.relpath(stack,os.path.dirname(os.path.realpath(__file__)))

NOTSET=0
DEBUG=10
def debug(message):
    basicConfig().debug(_path(inspect.stack()[1][1]),message)

INFO=20
def info(message):
    basicConfig().info(_path(inspect.stack()[1][1]),message)

WARNING=30
def warning(message):
    basicConfig().warning(_path(inspect.stack()[1][1]),message)

ERROR=40
def error(message):
    basicConfig().error(_path(inspect.stack()[1][1]),message)

CRITICAL=50
def critical(message):
    basicConfig().critical(_path(inspect.stack()[1][1]),message)

class configClass():
    _level=NOTSET
    _defaultLevel=_level

    def __new__(cls, *args, **kwargs):
        self = super().__new__(cls)
        if 'level' in kwargs:
            self._defaultLevel=kwargs['level']
            self._level=self._defaultLevel
        else:
            self._level=self._defaultLevel
        return self

# Singleton
class basicConfig(configClass):
    _instance = None
    def __new__(cls, *args, **kwargs):
        if 'force' in kwargs:
            if kwargs['force']==True:
                del kwargs['force']
                cls._instance = super().__new__(cls, *args, **kwargs)
                return cls._instance
        if cls._instance == None:
            cls._instance = super().__new__(cls, *args, **kwargs)
        return cls._instance

    def debug(self,relpath,message):
        if self._level <= DEBUG:
            print(str(relpath)+": "+"debug: "+str(message))

    def info(self,relpath,message):
        if self._level <= INFO:
            print(str(relpath)+": "+"info: "+str(message))

    def warning(self,relpath,message):
        if self._level <= WARNING:
            print(str(relpath)+": "+"WARN: "+str(message))

    def error(self,relpath,message):
        if self._level <= ERROR:
            print(str(relpath)+": "+"ERROR: "+str(message))

    def critical(self,relpath,message):
        if self._level <= CRITICAL:
            print(str(relpath)+": "+"CRITICAL: "+str(message))

def setLevel(level):
    basicConfig()._level=level

def disable(level):
    basicConfig()._level=level+1


# Copyright (C) 2017 Fassio Blatter

from PySide2.QtWidgets import QFileDialog,QInputDialog
from stopeight.util.editor.data import WaveData

import numpy as np
import sys

import stopeight.logging as log

def _getChannel(spf,channel_num=0,total_channels=1):
    #Extract Raw Audio from Wav File
    signal = spf.readframes(-1)
    datatype=None
    fconverter=lambda a:a
    if spf.getsampwidth()==1:
        #8bit unsigned
        datatype=np.uint8
        fconverter=lambda a: ((a/255.0) - 0.5) *2
    elif spf.getsampwidth()==2:
        #16bit signed
        datatype=np.int16
        fconverter=lambda a: a / 32767.0
    assert type(channel_num) is int
    assert type(total_channels) is int
    #left samples[0::2]
    #right samples[1::2]
    return fconverter(np.asarray(np.fromstring(signal, datatype)[channel_num::total_channels], dtype=np.float64)).view(WaveData)

def _selectChannel(spf):
    from stopeight.util.editor.runnable import EditorApp
    n = spf.getnchannels()
    channels = ['{}'.format(i) for i in range(n)]
    channel_num, ok = QInputDialog().getItem(EditorApp().window,"Select Channel","Channel:",channels,0,False)
    if ok:
        return int(channel_num),int(n)
    else:
        return 0,int(n)

def _open(filename):
    import wave
    from wave import open
    log.setLevel(log.INFO)
    spf = wave.open(filename,'r')
    if spf.getcomptype()!='NONE':
        raise Exception("Compressed WAV files are not supported.")
    channels = _selectChannel(spf)
    samples = _getChannel(spf,*channels)
    spf.close()
    log.debug("wave ndarray is "+str(type(samples)))
    log.info("Length of samples in track "+str(len(samples)))
    return samples

def open_WAV():
    log.setLevel(log.INFO)
    from stopeight.util.editor.runnable import EditorApp
    filename = QFileDialog.getOpenFileName(EditorApp().window)
    assert type(filename[0]) == str
    assert len(filename[0]) != 0
    log.info("Opening "+str(filename[0]))
    samples = _open(filename[0])
    return samples
open_WAV.__annotations__ = {'return': WaveData}



#!/usr/bin/env python

# Copyright (C) 2009-2012 Specific Purpose Software GmbH

from stopeight.comparator.lineMatch import LineMatchClass

import stopeight.logging as log
log.disable(log.CRITICAL)

class ShapeMatchClass(LineMatchClass):
    def __init__(self, compvec, toleranceDivider=2):
        LineMatchClass.__init__(self,compvec,toleranceDivider)

    def match(self,invec):
        '''
        This gives a boolean result for every line that entirely matches another line of the same length.
        '''
        #overwrite method in lineMatch for now
        #if we don't have SubSection matching,
        #and we don't have Directional matching,
        #it's only weighted (to be implemented: align from center not start)
        #this should be weighted, noreverse
        #is unweighted, reverse at the moment
        comparison = invec.copy()

        log.debug(dir(LineMatchClass))
        # this is being done if not instance of SubDivision
        if not LineMatchClass.validate_data(comparison,self.reference):
            raise Exception('data validation failed')

        comparison = self.scale(comparison)

        log.debug('Comparing:')
        log.debug(comparison)
        log.debug('To:')
        log.debug(self.reference)

        # all we need from here is comparison
        log.info('Moving comparison to point 0,0')
        inAtZero = comparison.moveToZero()
        log.debug(inAtZero)

        log.info('Moving comparison reversed to point 0,0')
        reverse = comparison.reverse()
        reverseAtZero = reverse.moveToZero()
        log.debug(reverseAtZero)

        forward = LineMatchClass.align(0,1,self.reference,inAtZero)
        backward = LineMatchClass.align(0,1,self.reference,reverseAtZero)

        if LineMatchClass.compare(self.tolerance,self.reference,forward) or LineMatchClass.compare(self.tolerance,self.reference,backward):
            log.info('We have a match!')
            return True
        else:
            log.info('No match.')
            return False

if __name__ == "__main__":
    from stopeight.comparator import vectorTools
    from numpy import array
    log.disable(log.NOTSET)
    log.info('Starting shapeMatch...')
    input = array([[20,20],[55,130],[80,60],[120,85],[200,10]]).view(vectorTools.NumpyLine)
    log.info('rendering input.svg')
    input.printVectors('input')
    dbrecord = array([[13,40],[50,125],[80,61],[123,79],[191,11]]).view(vectorTools.NumpyLine)
    log.info('rendering dbrecord.svg')
    dbrecord.printVectors('dbrecord')
    matcher = ShapeMatchClass(dbrecord)
    if matcher.match(input):
        print('Hooray!')



#!/usr/bin/env python

# Copyright (C) 2016 Specific Purpose Software GmbH

#############################################################################
##
## Copyright (C) 2013 Riverbank Computing Limited.
## Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
## All rights reserved.
##
## This file is part of the examples of PyQt.
##
## $QT_BEGIN_LICENSE:BSD$
## You may use this file under the terms of the BSD license as follows:
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are
## met:
##   * Redistributions of source code must retain the above copyright
##     notice, this list of conditions and the following disclaimer.
##   * Redistributions in binary form must reproduce the above copyright
##     notice, this list of conditions and the following disclaimer in
##     the documentation and/or other materials provided with the
##     distribution.
##   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
##     the names of its contributors may be used to endorse or promote
##     products derived from this software without specific prior written
##     permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
## $QT_END_LICENSE$
##
#############################################################################


from PySide2.QtCore import QPoint, QRect, QSize, Qt
from PySide2.QtGui import QImage, QPainter, QPen, qRgb
from PySide2 import QtWidgets
from PySide2.QtWidgets import (QColorDialog,
        QInputDialog)
from PySide2.QtPrintSupport import QPrintDialog, QPrinter

from PySide2.QtCore import QEvent

from stopeight.util.editor.runnable import EditorApp
from stopeight.util.editor.data import ScribbleData

import stopeight.logging as log

class ScribbleArea(QtWidgets.QDockWidget):
    def __init__(self, parent=EditorApp().window):
        super(ScribbleArea, self).__init__("ScribbleArea",parent)
        self.setMinimumSize(QSize(300, 256))

        self.setAttribute(Qt.WA_StaticContents)
        self.setAttribute(Qt.WA_AcceptTouchEvents)
        self.setAttribute(Qt.WA_TouchPadAcceptSingleTouchEvents)
        self.scribbling = False
        self.scribblingData=[]
        self.scribblingTop=self.scribblingBottom=self.scribblingLeft=self.scribblingRight=None
        self.scribblingMove=0
        self.scribblingDrawIndex = 0
        self.scribblingFingerCount = 0
        self.scribblingCurrentFinger=None

        self.myPenWidth = 1
        self.myPenColor = Qt.blue
        self.image = QImage()

        self.data = ScribbleData()

    def setPenColor(self, newColor):
        self.myPenColor = newColor

    def setPenWidth(self, newWidth):
        self.myPenWidth = newWidth

    def clearImage(self):
        self.image.fill(qRgb(255, 255, 255))
        self.update()

    def _input(self, x, y):
        self.scribblingData.append((x,y))
        if y<self.scribblingTop:
            self.scribblingTop=y
        if y>self.scribblingBottom:
            self.scribblingBottom=y
        if x<self.scribblingLeft:
            self.scribblingLeft=x
        if x>self.scribblingRight:
            self.scribblingRight=x

    def _press(self,event):
        self.clearImage()
        self.scribbling = True
        if (event.type() == QEvent.TouchBegin):
            self.scribblingFingerCount=len(event.touchPoints())
            self.scribblingCurrentFinger=len(event.touchPoints())-1
            self.scribblingTop=self.scribblingBottom=0
            self.scribblingLeft=self.scribblingRight=0
        elif (event.type() == QEvent.MouseButtonPress):
            self.scribblingTop=self.scribblingBottom=event.localPos().y()
            self.scribblingLeft=self.scribblingRight=event.localPos().x()
        elif (event.type() == QEvent.TabletPress):
            self.scribblingTop=self.scribblingBottom=event.posF().y()
            self.scribblingLeft=self.scribblingRight=event.posF().x()
        self.scribblingDrawIndex=0

    def _move(self, event):
        if (event.type() == QEvent.TouchUpdate):
            if (len(event.touchPoints())!=self.scribblingFingerCount):
                self.scribblingFingerCount=len(event.touchPoints())
                self.scribblingCurrentFinger=len(event.touchPoints())-1
        self.scribblingMove+=1
        if self.scribblingMove > 20:
            self._drawLines()
            self.scribblingMove=0

    def _release(self, event):
        self.scribbling = False
        if (event.type() == QEvent.TouchEnd):
            self.scribblingFingerCount=0
            if (len(event.touchPoints())!=self.scribblingFingerCount):
                self.scribblingCurrentFinger=len(event.touchPoints())-1
        self._drawLines()
        if type(self.data) is not type(None):
            del self.data
        self.data = ScribbleData(size=len(self.scribblingData))
        for i,v in enumerate(self.scribblingData):
            self.data[i]['coords'] = [v[0],v[1]]
        #don't assign new self.data = [], maybe self.data[:] = []
        if type(self.scribblingData) is not type(None):
            del self.scribblingData
        self.scribblingData = []

    def _cancel(self, event):
        self.scribbling = False
        if (event.type() == QEvent.TouchCancel):
            self.scribblingFingerCount=0
            if (len(event.touchPoints())!=self.scribblingFingerCount):
                self.scribblingCurrentFinger=len(event.touchPoints())-1
        if type(self.scribblingData) is not type(None):
            del self.scribblingData
        self.scribblingData=[]
        self.scribblingTop=self.scribblingBottom=self.scribblingLeft=self.scribblingRight=None
        self.scribblingMove=0
        self.scribblingDrawIndex = 0


    def _drawLines(self):
        painter = QPainter(self.image)
        if len(self.scribblingData)>self.scribblingDrawIndex+1:
            try:
                painter.setPen(QPen(self.myPenColor, self.myPenWidth, Qt.SolidLine,Qt.RoundCap, Qt.RoundJoin))
                for n in range(len(self.scribblingData)-1-self.scribblingDrawIndex):
                    painter.drawLine(QPoint(self.scribblingData[n+self.scribblingDrawIndex][0],self.scribblingData[n+self.scribblingDrawIndex][1]), QPoint(self.scribblingData[n+1+self.scribblingDrawIndex][0],self.scribblingData[n+1+self.scribblingDrawIndex][1]))
            finally:
                painter.end()
            rad = self.myPenWidth / 2 + 2
            self.update(QRect(QPoint(self.scribblingLeft,self.scribblingTop), QPoint(self.scribblingRight,self.scribblingBottom)).normalized().adjusted(-rad, -rad, +rad, +rad))
            self.scribblingDrawIndex=len(self.scribblingData)-1
            self.scribblingTop=self.scribblingBottom=self.scribblingData[-1][1]
            self.scribblingLeft=self.scribblingRight=self.scribblingData[-1][0]

    #todo: Remove try/catch in tableteventrealease
    #todo: Switch to hiResGlobalX; remainder: global - int(global); QWidget mapFromGlobal
    def event(self,event):
        if (event.type() == QEvent.MouseButtonPress):
            if event.button() == Qt.LeftButton:
                self._press(event)
                self._input(event.localPos().x(),event.localPos().y())
                return True           
        elif (event.type() == QEvent.MouseMove and self.scribbling):
            if (event.buttons() & Qt.LeftButton) and self.scribbling:
                self._input(event.localPos().x(),event.localPos().y())
                self._move(event)
                return True
        elif (event.type() == QEvent.MouseButtonRelease and self.scribbling):
            if (event.button() == Qt.LeftButton) and self.scribbling:
                self._input(event.localPos().x(),event.localPos().y())
                self._release(event)
                return True
        elif (event.type() == QEvent.TabletPress):
            self._press(event)
            self._input(event.posF().x(),event.posF().y())
            return True
        elif (event.type() == QEvent.TabletMove) and self.scribbling:
            self._input(event.posF().x(),event.posF().y())            
            self._move(event)
            return True
        elif (event.type() == QEvent.TabletRelease) and self.scribbling:
            self._input(event.posF().x(),event.posF().y())
            self._release(event)
            try:
                self.tablet_id=event.uniqueId()
            except:
                self.__dict__.__delattr__('tablet_id')
            return True
        elif (event.type() == QEvent.TouchBegin):
            self._press(event)
            self._input(event.touchPoints()[-1].pos().x(),event.touchPoints()[-1].pos().y())
            return True
        elif (event.type() == QEvent.TouchUpdate and self.scribbling):
            self._input(event.touchPoints()[-1].pos().x(),event.touchPoints()[-1].pos().y())
            self._move(event)
            return True
        elif (event.type() == QEvent.TouchEnd and self.scribbling):
            self._input(event.touchPoints()[-1].pos().x(),event.touchPoints()[-1].pos().y())
            self._release(event)
            return True
        elif (event.type() == QEvent.TouchCancel and self.scribbling):
            self._input(event.touchPoints()[-1].pos().x(),event.touchPoints()[-1].pos().y())
            self._cancel(event)
            return True
        return super().event(event)

    def identify(self)->str:
        import os
            
        #if (os.getcwd()).endswith('stopeight'):
        #    if (self.select.module_name=='stopeight.util.editor.modules.file'):
        if hasattr(self,'tablet_id'):
            sub = str(self.tablet_id)
        else:
            sub = 'MouseData'
        return sub

    def paintEvent(self, event):
        painter = QPainter(self)
        dirtyRect = event.rect()
        try:
            painter.drawImage(dirtyRect, self.image, dirtyRect)
        finally:
            painter.end()

    def resizeEvent(self, event):
        if self.width() > self.image.width() or self.height() > self.image.height():
            newWidth = max(self.width() + 128, self.image.width())
            newHeight = max(self.height() + 128, self.image.height())
            self.resizeImage(self.image, QSize(newWidth, newHeight))
            self.update()
        super(ScribbleArea, self).resizeEvent(event)

    def __call__(self, data, color=Qt.blue, clear=True):
        assert type(self.data) is ScribbleData
        if clear:
            self.clearImage()
        #self.data=data
        #for d,c in self.data,self.colors:
        #self.scribble.plot(d,c)
        log.debug("Drawing "+str(len(data))+" lines")
        painter = QPainter(self.image)
        try:
            painter.setPen(QPen(color, self.myPenWidth, Qt.SolidLine,Qt.RoundCap, Qt.RoundJoin))
            for first,second in zip(data,data[1:]):
                painter.drawLine(first['coords'][0],first['coords'][1],second['coords'][0],second['coords'][1])
        finally:
            painter.end()
        self.update()
    __call__.__annotations__ = {'data': ScribbleData}

    def resizeImage(self, image, newSize):
        if image.size() == newSize:
            return

        newImage = QImage(newSize, QImage.Format_RGB32)
        newImage.fill(qRgb(255, 255, 255))
        painter = QPainter(newImage)
        try:
            painter.drawImage(QPoint(0, 0), image)
        finally:
            painter.end()
        self.image = newImage

    def print_(self):
        printer = QPrinter(QPrinter.HighResolution)

        printDialog = QPrintDialog(printer, self)
        if printDialog.exec_() == QPrintDialog.Accepted:
            painter = QPainter(printer)
            rect = painter.viewport()
            size = self.image.size()
            size.scale(rect.size(), Qt.KeepAspectRatio)
            painter.setViewport(rect.x(), rect.y(), size.width(), size.height())
            painter.setWindow(self.image.rect())
            try:
                painter.drawImage(0, 0, self.image)
            finally:
                painter.end()

    def penColor(self):
        return self.myPenColor

    def penWidth(self):
        return self.myPenWidth

#    def penColor(self):
#        newColor = QColorDialog.getColor(self.penColor())
#        if newColor.isValid():
#            self.setPenColor(newColor)

#    def penWidth(self):
#        newWidth, ok = QInputDialog.getInt(self, "Scribble",
#                "Select pen width:", self.penWidth(), 1, 50, 1)
#        if ok:
#            self.setPenWidth(newWidth)




# Copyright (C) 2009-2016 Specific Purpose Software GmbH

from numpy import array,pi,cos,sin
from numpy import sqrt,exp
#from SVG import Scene
#from numpy.oldnumeric.ma import dot
from numpy import dot

# This matrix is a: Column first style

_deg_to_rad = lambda deg: 2 * pi / (360.0/deg)

def idmatrix():
    return array([[1, 0, 0], [0, 1, 0], [0, 0, 1]])

class Matrix:
    def __init__(self,translation=[0,0],rotation=0):
        self.x = translation[0]
        self.y = translation[1]
        self.rot = _deg_to_rad(rotation)

#def scale(a):
#    m = array([[a[0],0,0],[0,a[1],0],[0,0,1]])
#    return m

# row first
def transform(a, m):
    b = array([0,0])
    b[0] = cos(m.rot) * a[0] + -sin(m.rot) * a[1] + m.x
    b[1] = sin(m.rot) * a[0] + cos(m.rot) * a[1] + m.y
    return b

#def center(a):
#    return transform(a,translate(array([0,0])))

if __name__ == '__main__':
    a = array([30,30])
    #m = idmatrix()
    print ('rotate vector with matrix:')
    #m = dot(translate(array([10,15])),m)
    #m = dot(scale(array([0.5,0.5])),m)
    #m = dot(rotate(-90),m)
    m = Matrix([10,15],-90)
    result = transform(a,m)
    print(a)
    print(result)
#    scene = Scene('matrixTools',255,255)
#    scene.add(Line(center(array([0,0])),center(a)))
#    scene.add(Line(center(array([0,0])),center(result)))
#    scene.write_svg()
#    scene.display()
#    return



#!/usr/bin/env python

import os
_include_dirs=[
    os.path.join('stopeight-clibs','cmake-git-version-tracking','better-example'),
]
_qt5_include_dirs=_include_dirs

from pybind11.setup_helpers import Pybind11Extension, build_ext
from setuptools import setup, find_namespace_packages

setup( use_scm_version=True,
       packages=find_namespace_packages(include=['stopeight.*']),
       ext_modules = [
           Pybind11Extension(
               'stopeight.grapher',
               [os.path.join('stopeight-clibs','grapher','python','IFPyGrapher.cpp')],
               include_dirs=_include_dirs + [
                   os.path.join('stopeight-clibs','include'),
                   os.path.join('stopeight-clibs','grapher')
               ],
               libraries=['stopeight-clibs-grapher',
               'stopeight-clibs-matrix'],
               language='c++',
               optional=False
           ),
           Pybind11Extension(
               'stopeight.matrix',
               [os.path.join('stopeight-clibs','matrix','python','IFPyMatrix.cpp')],
               include_dirs=_include_dirs + [
                   os.path.join('stopeight-clibs','include'),
                   os.path.join('stopeight-clibs','matrix')
               ],
               libraries=['stopeight-clibs-matrix'],
               language='c++',
               optional=False
           ),
           Pybind11Extension(
               'stopeight.analyzer',
               [os.path.join('stopeight-clibs','analyzer','python','IFPyAnalyzer.cpp')],
               include_dirs=_include_dirs + [
                   os.path.join('stopeight-clibs','include'),
                   os.path.join('stopeight-clibs','analyzer')
               ],
               libraries=['stopeight-clibs-analyzer'],
               language='c++',
               optional=True
           ),
           Pybind11Extension(
              'stopeight.legacy',
               [os.path.join('stopeight-clibs','legacy','python','interfacepython.cpp'),
               ],
               include_dirs=_qt5_include_dirs + [
                   os.path.join('stopeight-clibs','legacy/include'),
                   os.path.join('stopeight-clibs','legacy','python')
               ],
               libraries=['stopeight-clibs-legacy','Qt5Core'],#Qt5Core needed for old wrapper
               language='c++',
               optional=True
              ),
            Pybind11Extension(
              'stopeight.getters',
               [os.path.join('stopeight-clibs','legacy','python','IFPyGetters.cpp'),
               os.path.join('stopeight-clibs','legacy','python','IFPyShared.cpp')],
               include_dirs=_qt5_include_dirs + [
                   os.path.join('stopeight-clibs','legacy/include'),
                   os.path.join('stopeight-clibs','legacy','python')
               ],
               libraries=['stopeight-clibs-legacy','stopeight-clibs-matrix','Qt5Core'],#Qt5Core needed for old wrapper
               language='c++',
               optional=True
              ),
            Pybind11Extension(
              'stopeight.finders',
               [os.path.join('stopeight-clibs','legacy','python','IFPyFinders.cpp'),
               os.path.join('stopeight-clibs','legacy','python','IFPyShared.cpp')],
               include_dirs=_qt5_include_dirs + [
                   os.path.join('stopeight-clibs','legacy/include'),
                   os.path.join('stopeight-clibs','legacy','python')
               ],
               libraries=['stopeight-clibs-legacy','stopeight-clibs-matrix','Qt5Core'],#Qt5Core needed for old wrapper
               language='c++',
               optional=True
              ),
       ],
       cmdclass={'build_ext': build_ext},
)



def show(filename='parser.out.svg'):
    import sys

    sys.path.append('..\PyQt5\examples\painting\svgviewer')

    from PyQt5.QtWidgets import QApplication

    app = QApplication(sys.argv)

    import svgviewer

    window = svgviewer.MainWindow()
    window.openFile(filename)
    window.show()
    sys.exit(app.exec_())

